//////////////////////////////////////////////////////////////////////////////////////////
// Project description
// 
// Name: Beamer Latency Measurement Tool (BLMT)
// Purpose: Measure the latency of the beamer compared to a regular monitor
//
// Author: Laurence Muller (aka Falcon4ever)
// Contact: info@falcon4ever.com
// Site: www.multigesture.net
// Licensetype: GNU General Public License (GPL)
// Last update: 3 November 2007
//
// Changes:
// v0.1 - Initial release. (3 November 2007)
// v0.2 - Fixed Mac OS X compatibility + Custom screenres support (Pawel 'solydzajs' Solyga)
//
// Todo:
// - Got suggestions?
//
//
// Tool based on paper:
//	Real-Time Interaction with a Humanoid Avatar in an Immersive Table Tennis Simulation
//	http://portal.acm.org/citation.cfm?id=1187756
//
//////////////////////////////////////////////////////////////////////////////////////////

//////////////////////////////////////////////////////////////////////////////////////////
// Include files
// 
#include "main.h"

// Used namespaces
using namespace std;

//////////////////////////////////////////////////////////////////////////////////////////
// Please change the screen properties here
//
// 1024 / 60 = 17.0666667
/*
const int scn_x = 1024;
const int scn_y = 768;
const int gridsize_s = 17;
const float fgridsize_s = 17.;
*/
//
// 800 / 60 = 13.3333333
/*
const int scn_x = 800;
const int scn_y = 600;
const int gridsize_s = 13;
const float fgridsize_s = 13.;
*/
//////////////////////////////////////////////////////////////////////////////////////////

int screen_width, screen_height, bitdepth, refreshrate;
int font = (int)(intptr_t)GLUT_BITMAP_8_BY_13;

int gridsize_s;
float fgridsize_s;

//////////////////////////////////////////////////////////////////////////////////////////
// Main
// 
int main(int argc, char **argv) 
{
	char displaymode[24];
	
	if(argc != 4)
	{
		print_usage(argv);
	} 
	
	// Setup glut
	glutInit(&argc, argv);
	glutInitDisplayMode(GLUT_DEPTH | GLUT_DOUBLE | GLUT_RGBA);
	
	// Convert parsed values
	screen_width = atoi(argv[1]);
	screen_height = atoi(argv[2]);
	bitdepth = 16;
	refreshrate = atoi(argv[3]);
	
	gridsize_s = screen_width / 60;
	fgridsize_s = gridsize_s;
	
	// Set display mode
	sprintf(displaymode, "%dx%d:%d@%d", screen_width, screen_height, bitdepth, refreshrate);
	glutGameModeString(displaymode);
	
	if (glutGameModeGet(GLUT_GAME_MODE_POSSIBLE)) 
	{
		glutEnterGameMode();			
	}
	else 
	{
		printf("The select mode is not available\n");
		exit(1);	
	}

	opengl_init();

	glutDisplayFunc(render_scene);
	glutIdleFunc(render_scene);
	glutKeyboardFunc(keyboard_input);
	glutMainLoop();	// Endless loop

	return 0;
}

void print_usage(char **argv)
{
	fprintf(stderr,"Usage: %s Width Height RefreshRate\n", argv[0]);
	fprintf(stderr,"Example: %s 1024 768 60\n");
	exit(0);
}

// Init opengl defaults
void opengl_init()
{
	glClearColor(0.0f, 0.0f, 0.5f, 0.0f);		// Blue Background	
	glClearDepth(1.0f);							// Depth Buffer Setup	

	glEnable(GL_BLEND); 
	glDisable(GL_DEPTH_TEST);
	glBlendFunc(GL_SRC_ALPHA, GL_ONE);			// Blending Function For Translucency Based On Source Alpha Value
}

// Handle keyboard input
void keyboard_input(unsigned char key, int x, int y)
{
	switch (key) 
	{
		case 'q':
			exit(0);
		break;	
		
		case 27:  /*  Escape Key */			
			exit(0);
		break;

		default:
			break;
	}
}

void render_scene()
{
	// Clear The Screen And The Depth Buffer
	glClear(GL_COLOR_BUFFER_BIT | GL_DEPTH_BUFFER_BIT);

	// Set view to Ortho2D for easy 2D positioning
	glViewport(0, 0, screen_width, screen_height);
	glMatrixMode(GL_PROJECTION);
	glLoadIdentity();
	gluOrtho2D(0, screen_width, screen_height, 0);
	glMatrixMode(GL_MODELVIEW);

	// Render grid (background)
	render_grid();
	debug_info();
	render_bar();

	// Swap buffers!
	glFlush();
	glutSwapBuffers();
}

void render_bar()
{
	static int state=0;

	float x = (float)state * fgridsize_s;
	float xx = x + fgridsize_s; 

	glColor3f(0.0f, 1.0f, 0.0f);	
	glBegin(GL_QUADS);						// Draw A Quad
		glVertex2f( x, 1);				// Top Left
		glVertex2f( xx, 1);				// Top Right
		glVertex2f( xx, 799);				// Bottom Right
		glVertex2f( x, 799);				// Bottom Left
	glEnd();

	++state;
	if(state==60)
		state=0;
}
/*
	Note:
	coordinate system is like this (screen of 1024 x 768)
	[Windows]
	Range x = 0 ... 1023
	Range y = 1 ... 768
   0,1   1023,1
	+---+
	|   |
	+---+
 0,768   1023,768
*/
void render_grid()
{	
	#if defined(WIN32)
	int x_first = 1;
	#else
	int x_first = 0;
	#endif

	glBegin(GL_LINES);
		glColor3f(1.0f, 1.0f, 1.0f);
		for(int x=x_first; x <= screen_width; x += gridsize_s)
		{
			glVertex2f(x, 250);
			glVertex2f(x, screen_height-250);			
		}
	glEnd();		
}

void debug_info()
{	
	char buffer[64];	
	static int frame=0, time, timebase=0;
	static float currentfps;

	++frame;
	time = glutGet(GLUT_ELAPSED_TIME);

	if (time - timebase > 1000) 
	{		
		currentfps = frame * 1000.0 / (time - timebase);
		timebase = time;		
		frame = 0;
	}

	glColor4f(1.0, 0.0, 0.0, 1.0);
	sprintf(buffer, "Beamer Latency Measurement Tool");
	renderBitmapCharacter(10, 24, (void *)(intptr_t)font, buffer);
	sprintf(buffer, "Average FPS: %4.2f", currentfps);   
 	renderBitmapCharacter(10, 40, (void *)(intptr_t)font, buffer);
	sprintf(buffer, "Display settings: %dx%d %db @ %dHz", screen_width, screen_height, bitdepth, refreshrate);   	
	renderBitmapCharacter(10, 56, (void *)(intptr_t)font, buffer);	
}

void renderBitmapCharacter(int x, int y, void *font, char *string)
{			
	char *c;
	glRasterPos2i(x, y);
	
	for (c=string; *c != '\0'; c++) 
		glutBitmapCharacter(font, *c);					
}
